/*
 * (C) COPYRIGHT 2014 CRZ
 *
 * File Name : test_sensor_HTU21.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 07/30/2014
 */

/* includes */

#include "hw_config.h"
#include "global_types.h"
#include "uart_queue.h"

/* functions */

bool UartQueue_Serial_Is_Empty(void);
uint16_t UartQueue_Serial_DeQueue(void);

/*
 * device operation
 */

static bool g_print_menu_control_flag = TRUE;

static void print_menu_command(void)
{
    if(TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("1> Output port 0 On\n");
        printf("2> Output port 0 Off\n");
        printf("3> Output port 1 On\n");
        printf("4> Output port 1 Off\n");
        printf("5> Output port 2 On\n");
        printf("6> Output port 2 Off\n");
        printf("7> Output port 3 On\n");
        printf("8> Output port 4 Off\n");
        printf("---------------------\n");
        printf("9> Read Input ports\n");
        printf("a> Read DIP switch ports\n");
        printf("b> Read Output ports\n");
        printf("---------------------\n");
        printf("x> quit\n\n");
    }

    g_print_menu_control_flag = FALSE;
}

static void actuator_port_setup(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    /* DIP Switch setup - Input */

    GPIO_InitStructure.GPIO_Pin   = GPIO_MCU_MIO_0_PIN | GPIO_MCU_MIO_1_PIN | GPIO_MCU_MIO_3_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_MCU_MIO, &GPIO_InitStructure);

    /* PA setup - Input */

    GPIO_InitStructure.GPIO_Pin   = GPIO_MCU_PA_0_PIN | GPIO_MCU_PA_1_PIN | GPIO_MCU_PA_2_PIN | GPIO_MCU_PA_3_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_MCU_PA, &GPIO_InitStructure);

    /* PA setup - Output */

    GPIO_InitStructure.GPIO_Pin   = GPIO_MCU_PA_4_PIN | GPIO_MCU_PA_5_PIN | GPIO_MCU_PA_6_PIN | GPIO_MCU_PA_7_PIN;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
    GPIO_Init(GPIO_MCU_PA, &GPIO_InitStructure);

    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN | GPIO_MCU_PA_5_PIN | GPIO_MCU_PA_6_PIN | GPIO_MCU_PA_7_PIN);
}

void Actuator_Output_Port_0_On(void)
{
    GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
}

void Actuator_Output_Port_0_Off(void)
{
    GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
}

void Actuator_Output_Port_0_Toggle(void)
{
    GPIO_WriteBit(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN,
                  (BitAction)(1-GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN)));
}

void Test_Actuator(void)
{
    uint8_t ch;

    DEBUG_MSG_FUNC_START;

    actuator_port_setup();
    printf("actuator_port_setup() done\n");

    while(1)
    {
        print_menu_command();

        if(FALSE == UartQueue_Serial_Is_Empty())
        {
            ch = UartQueue_Serial_DeQueue();
            printf("%c is selected\n\n", (char)ch);

            switch((char)ch)
            {
            case '1': Actuator_Output_Port_0_On();   break;
            case '2': Actuator_Output_Port_0_Off(); break;
            case '3': GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN);   break;
            case '4': GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN); break;
            case '5': GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN);   break;
            case '6': GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN); break;
            case '7': GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN);   break;
            case '8': GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN); break;

            case '9':
                printf("Input Port P0: %d, P1: %d, P2: %d, P3: %d\n",
                    GPIO_ReadInputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_0_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_1_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_2_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_3_PIN));
                break;

            case 'a':
                printf("Dip Switch Port D4: %d, D5: %d, D6: %d\n",
                    GPIO_ReadInputDataBit(GPIO_MCU_MIO, GPIO_MCU_MIO_0_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_MIO, GPIO_MCU_MIO_1_PIN),
                    GPIO_ReadInputDataBit(GPIO_MCU_MIO, GPIO_MCU_MIO_3_PIN));
                break;

            case 'b':
                printf("Output Port P0: %d, P1: %d, P2: %d, P3: %d\n",
                    GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN),
                    GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_5_PIN),
                    GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_6_PIN),
                    GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_7_PIN));
                break;
            }

            g_print_menu_control_flag = TRUE;

            if('x' == (char)ch)
            {
                break;
            }
        }
    }

    return;
}

typedef enum
{
    MCU_COMM_RX_STATE_INIT               = 0,
    MCU_COMM_RX_STATE_CONTROL_T_RECEIVED = 1,
    MCU_COMM_RX_STATE_CONTROL_W_RECEIVED = 2,
    MCU_COMM_RX_STATE_CONTROL_L_RECEIVED = 3,
    MCU_COMM_RX_STATE_REQ_STS_RECEIVED   = 4,
} Mcu_Comm_Rx_State_E;

Mcu_Comm_Rx_State_E g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
bool g_actuator_blink_state = FALSE;

void control_state_zigbee_comm(uint8_t ch)
{
    switch(g_rx_curr_state)
    {
    case MCU_COMM_RX_STATE_INIT:
        if('T' == (char)ch)
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_CONTROL_T_RECEIVED;
        }
        break;

    case MCU_COMM_RX_STATE_CONTROL_T_RECEIVED:
        if('W' == (char)ch)
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_CONTROL_W_RECEIVED;
        }
        else
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
        }
        break;

    case MCU_COMM_RX_STATE_CONTROL_W_RECEIVED:
        if('L' == (char)ch)
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_CONTROL_L_RECEIVED;
        }
        else
        {
            g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
        }
        break;

    case MCU_COMM_RX_STATE_CONTROL_L_RECEIVED:
        switch((char)ch)
        {
        case '0': /* Off */
            g_actuator_blink_state = FALSE;
            GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
            break;
        case '1': /* On */
            g_actuator_blink_state = FALSE;
            GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
            break;
        case '2': /* Blink */
            g_actuator_blink_state = TRUE;
            break;
        default:
            break;
        }

        g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
        break;

    case MCU_COMM_RX_STATE_REQ_STS_RECEIVED:
        {
            uint8_t ch_1_data;

            if(TRUE == g_actuator_blink_state)
            {
                ch_1_data = 2;
            }
            else
            {
                ch_1_data = GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
            }

            UartComm_SendData_OneChar('0' + ch_1_data);
            UartComm_SendData_OneChar(0);
            UartComm_SendData_OneChar(0);

            g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
        }
        break;
    default:
        break;
    }
}

void Test_Actuator_ZigBee_Comm(void)
{
    uint8_t ch;

    DEBUG_MSG_FUNC_START;

    actuator_port_setup();
    printf("actuator_port_setup() done\n");

    while(1)
    {
        if(FALSE == UartQueue_Comm_Is_Empty())
        {
            ch = UartQueue_Comm_DeQueue();
            printf("%c is received\n\n", (char)ch);

            switch((char)ch)
            {
            case 'S':
                {
                    uint8_t ch_1_data;

                    if(TRUE == g_actuator_blink_state)
                    {
                        ch_1_data = 2;
                    }
                    else
                    {
                        ch_1_data = GPIO_ReadOutputDataBit(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
                    }

                    UartComm_SendData_OneChar(ch_1_data);
                    UartComm_SendData_OneChar(0);
                    UartComm_SendData_OneChar(0);

                    g_rx_curr_state = MCU_COMM_RX_STATE_INIT;
                }
                break;

            case '0': /* Off */
                g_actuator_blink_state = FALSE;
                GPIO_ResetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
                break;

            case '1': /* On */
                g_actuator_blink_state = FALSE;
                GPIO_SetBits(GPIO_MCU_PA, GPIO_MCU_PA_4_PIN);
                break;

            case '2': /* Blink */
                g_actuator_blink_state = TRUE;
                break;
            }

//            control_state_zigbee_comm(ch);
        }

    }
}

